KEY_PRESS = 0

MOUSE_DOWN = 1
MOUSE_UP = 2
MOUSE_DOUBLE_CLICK = 3
MOUSE_MOVE = 5

SCROLL_DOWN = 6
SCROLL_UP = 7
SCROLL_STEP = 1

CTRL = 'ctrl'
SHIFT = 'shift'
ALT = 'alt'

MODIFIER_KEYS = (CTRL, SHIFT, ALT,)

MODIFIER_KEYS_PRESS_DELAY = .4
EVENTS_DELAY = .05

LEFT = "left"
MIDDLE = "middle"
RIGHT = "right"

QUALITY = 80

HIGH_SCALE = 1
LOW_SCALE = 2



import asyncio
import pickle
import sys
from threading import Thread

import cv2
import numpy as np
import websockets
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap, QImage
from PyQt5.QtWidgets import QMainWindow, QApplication, QLabel

from . import constant

key_translate = {}
for i in dir(Qt):
    data = i.split('_')
    if len(data) == 2:
        key_translate[getattr(Qt, i)] = data[1]

key_translate[16777249] = constant.CTRL
key_translate[16777248] = constant.SHIFT
key_translate[16777251] = constant.ALT


class EventList(list):
    def append(self, __object) -> None:
        if __object not in self:
            super().append(__object)


event_list = EventList()


async def connect(host, port, username, password, quality):
    global event_list

    async with websockets.connect(f"ws://{username}:{password}@{host}:{port}",
                                  max_size=None) as websocket:
        await websocket.send(quality)
        while True:
            events, event_list = event_list, EventList()

            await websocket.send(str(events))

            box = await websocket.recv()
            frame = await websocket.recv()
            w.update_frame(frame, box)


def start_client(app, host, port, username, password, quality):
    try:
        if sys.version_info >= (3, 7):
            import uvloop
            uvloop.install()

        asyncio.run(connect(host, port, username, password, quality))

    except Exception as e:
        print(e)
        app.exit()


class MainWindow(QMainWindow):

    def __init__(self):
        super(MainWindow, self).__init__()
        self.title = "DeskView"
        self.setMouseTracking(True)

        self.setWindowTitle(self.title)
        self.label = QLabel(self)
        self.current_frame = None
        self.current_width = None
        self.current_height = None
        self.setMouseTracking(True)

    @staticmethod
    def get_button(e):
        if e.button() == Qt.LeftButton:
            return constant.LEFT
        elif e.button() == Qt.RightButton:
            return constant.RIGHT
        elif e.button() == Qt.MiddleButton:
            return constant.MIDDLE

    # Mouse Events

    def mousePressEvent(self, e) -> None:
        event_list.append((
            constant.MOUSE_DOWN, (
                e.x() / self.current_width, e.y() / self.current_height,
                self.get_button(e))))

    def mouseReleaseEvent(self, e) -> None:
        event_list.append((
            constant.MOUSE_UP, (
                e.x() / self.current_width, e.y() / self.current_height,
                self.get_button(e))))

    def mouseDoubleClickEvent(self, e) -> None:
        event_list.append((
            constant.MOUSE_DOUBLE_CLICK, (
                e.x() / self.current_width, e.y() / self.current_height,
                self.get_button(e))))

    def wheelEvent(self, e):
        if e.angleDelta().y() > 0:
            event = constant.SCROLL_UP
        else:
            event = constant.SCROLL_DOWN

        event_list.append((event, constant.SCROLL_STEP))

    # Key Events

    def keyPressEvent(self, e):
        try:
            key = chr(e.key())
        except:
            key = key_translate.get(e.key())
        finally:
            event_list.append((constant.KEY_PRESS, key))

    def update_label_size(self) -> None:
        width, height = int(w.width()), int(w.height())
        if (self.current_width, self.current_height) != (width, height):
            self.current_width, self.current_height = width, height
            self.label.setFixedHeight(self.current_height)
            self.label.setFixedWidth(self.current_width)

    def update_frame(self, frame, box):
        self.update_label_size()

        if not (box and frame):
            return

        frame_cv2 = cv2.cvtColor(
            np.array(pickle.loads(frame)),
            cv2.COLOR_BGRA2BGR
        )

        if self.current_frame is not None:
            x_min, y_min, x_max, y_max = eval(box)
            self.current_frame[y_min: y_max, x_min: x_max] = frame_cv2
        else:
            self.current_frame = frame_cv2

        height, width = self.current_frame.shape[:2]

        self.label.setPixmap(
            QPixmap(
                QImage(self.current_frame, width, height,
                       QImage.Format_RGB888).rgbSwapped()
            ).scaled(self.current_width, self.current_height))


def main():
    global w
    import argparse

    parser = argparse.ArgumentParser()

    parser.add_argument('-H', '--host', metavar='ADDRESS', default='0.0.0.0',
                        help='Specify alternate connection address')

    parser.add_argument('--port', action='store',
                        default=8765, type=int,
                        nargs='?',
                        help='Specify alternate port [default: 8765]')

    parser.add_argument('--username', '-u', default='username')
    parser.add_argument('--password', '-p', default='password')

    parser.add_argument('--quality', '-q', default='high',
                        help="""high, low""")

    args = parser.parse_args()

    print(f'Connecting Server {args.host}:{args.port}')

    app = QApplication(sys.argv)
    w = MainWindow()
    w.setGeometry(640, 100, 800, 500)

    Thread(target=start_client, daemon=True, args=(app,
                                                   args.host,
                                                   args.port,
                                                   args.username,
                                                   args.password,
                                                   args.quality,
                                                   )).start()
    w.show()
    app.exit(app.exec_())


if __name__ == '__main__':
    main()



import asyncio
import base64
import http
import os
import pickle
import sys
from queue import Queue
from threading import Thread
from time import sleep

import cv2
import mss
import numpy
import numpy as np
import pyautogui
import websockets

from . import constant

pyautogui.FAILSAFE = False

WIDTH, HEIGHT = pyautogui.size()

events_queue = Queue()

AUTHORIZATION = None

sys.stderr = open(os.devnull, 'a')


class FrameService:
    def __init__(self, quality):
        self.scale = getattr(constant, quality.upper() + '_SCALE')
        self.last_frame_gray = None
        self.sct = mss.mss()
        self.monitor = self.sct.monitors[0]

    def screenshot(self):
        """ Handle with screenshot """
        return self.sct.grab(self.monitor)

    def resize(self, img):
        height, width = img.shape[:2]
        return cv2.resize(img,
                          (int(width / self.scale), int(height / self.scale)))

    @staticmethod
    def convert_to_cv2(sct_img):
        return cv2.cvtColor(numpy.array(sct_img), cv2.COLOR_BGRA2BGR)

    @staticmethod
    def convert_to_gray(cv2_img):
        return cv2.cvtColor(cv2_img, cv2.COLOR_BGR2GRAY)

    @staticmethod
    def get_bounding_box(gray_frame_1, gray_frame_2):
        es = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (9, 4))
        diff = cv2.absdiff(gray_frame_1, gray_frame_2)
        thresh = cv2.threshold(diff, 0, 255, cv2.THRESH_BINARY)[1]
        diff = cv2.dilate(thresh, es, iterations=2)

        contours, hierarchy = cv2.findContours(
            diff.copy(),
            cv2.RETR_LIST,
            cv2.CHAIN_APPROX_NONE
        )

        if not contours:
            return None

        boundings_x = []
        boundings_y = []
        for contour in contours:
            x, y, w, h = cv2.boundingRect(contour)

            boundings_x.append(x)
            boundings_x.append(x + w)
            boundings_y.append(y)
            boundings_y.append(y + h)

        x_min = min(boundings_x)
        y_min = min(boundings_y)
        x_max = max(boundings_x)
        y_max = max(boundings_y)
        return x_min, y_min, x_max, y_max

    @staticmethod
    def crop(frame, box):
        x_min, y_min, x_max, y_max = box
        return frame[y_min: y_max, x_min: x_max]

    def get(self):
        if self.last_frame_gray is None:
            ss = self.screenshot()
            frame = self.convert_to_cv2(ss)
            frame = self.resize(frame)
            self.last_frame_gray = self.convert_to_gray(frame)
            height, width = frame.shape[:2]
            return frame, (0, width, 0, height)

        ss = self.screenshot()
        frame = self.convert_to_cv2(ss)
        frame = self.resize(frame)
        frame_gray = self.convert_to_gray(frame)

        box = self.get_bounding_box(self.last_frame_gray, frame_gray)

        if not box:
            return None, None

        self.last_frame_gray = frame_gray
        frame_cropped = self.crop(frame, box)

        return frame_cropped, box


def delay_press(key):
    pyautogui.keyDown(key)
    sleep(constant.MODIFIER_KEYS_PRESS_DELAY)
    pyautogui.keyUp(key)


def events_handler():
    while True:
        sleep(constant.EVENTS_DELAY)

        while not events_queue.empty():

            e, c = events_queue.get()

            # Mouse

            if constant.MOUSE_DOWN == e:
                pyautogui.mouseDown(x=c[0] * WIDTH,
                                    y=c[1] * HEIGHT,
                                    button=c[2])

            elif constant.MOUSE_UP == e:
                pyautogui.mouseUp(x=c[0] * WIDTH,
                                  y=c[1] * HEIGHT,
                                  button=c[2])

            elif constant.MOUSE_DOUBLE_CLICK == e:
                pyautogui.doubleClick(x=c[0] * WIDTH,
                                      y=c[1] * HEIGHT,
                                      button=c[2])

            elif constant.MOUSE_MOVE == e:
                pyautogui.moveTo(x=c[0] * WIDTH,
                                 y=c[1] * HEIGHT,
                                 button=c[2])

            elif constant.SCROLL_DOWN == e:
                pyautogui.scroll(-c)

            elif constant.SCROLL_UP == e:
                pyautogui.scroll(c)

            # Keyboard

            elif constant.KEY_PRESS == e and c in constant.MODIFIER_KEYS:
                Thread(target=delay_press, args=(c,), daemon=True)

            elif constant.KEY_PRESS == e:
                pyautogui.press(e)


Thread(target=events_handler, daemon=True).start()


def get_basic_auth(username, password):
    user_pass = f'{username}:{password}'
    basic_credentials = base64.b64encode(user_pass.encode()).decode()
    return f'Basic {basic_credentials}'


async def screen_handler(websocket, path, *args, **kwargs):
    remote_ip = websocket.remote_address[0]
    print(f'Connected: {remote_ip}')

    quality = await websocket.recv()
    frame_service = FrameService(quality)

    while True:
        events = eval(await websocket.recv())

        for event in events:
            events_queue.put(event)

        frame_cropped, box = frame_service.get()

        if box:
            await websocket.send(bytes(str(box).encode()))
            await websocket.send(pickle.dumps(np.array(frame_cropped)))
        else:
            await websocket.send('')
            await websocket.send('')


class BasicAuthServerProtocol(websockets.WebSocketServerProtocol):

    async def process_request(self, path, request_headers):
        try:
            authorization = request_headers['Authorization']
        except KeyError:
            return http.HTTPStatus.UNAUTHORIZED, [], b'Missing credentials\n'
        if authorization != AUTHORIZATION:
            return http.HTTPStatus.FORBIDDEN, [], b'Incorrect credentials\n'


async def serve(bind="0.0.0.0", port=8765):
    async with websockets.serve(
            screen_handler,
            bind,
            port,
            max_size=None,
            create_protocol=BasicAuthServerProtocol):
        await asyncio.Future()


def main():
    global AUTHORIZATION
    import argparse

    parser = argparse.ArgumentParser()

    parser.add_argument('--bind', '-b', metavar='ADDRESS', default='0.0.0.0',
                        help='Specify alternate bind address '
                             '[default: all interfaces]')

    parser.add_argument('--port', action='store',
                        default=8765, type=int,
                        nargs='?',
                        help='Specify alternate port [default: 8765]')

    parser.add_argument('--username', '-u', default='username')
    parser.add_argument('--password', '-p', default='password')

    args = parser.parse_args()

    bind, port = args.bind, args.port

    AUTHORIZATION = get_basic_auth(args.username, args.password)

    print(f'Server Starting {bind}:{port}')

    if sys.version_info >= (3, 7):
        import uvloop

        uvloop.install()
        loop = asyncio.get_event_loop()
        loop.run_until_complete(serve(bind, port))
    else:
        asyncio.run(serve(bind, port))


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

requires = [
    'numpy==1.20.3; python_version >= "3"',
    'mss==6.1.0; python_version >= "3"',
    'websockets==10.0; python_version >= "3"',
    'asyncio==3.4.3; python_version >= "3.3"',
    'uvloop==0.16.0; python_version >= "3.7"',
    'PyQt5==5.15.4; python_version > "3"',
    'pyautogui==0.9.53; python_version >= "3"',
    'opencv-python-headless==4.5.3.56; python_version >= "3"',

]

with open("README.rst", "r", encoding="utf8") as f:
    readme = f.read()

setup(
    name='deskviewer',
    version='0.0.1',
    package_dir={'deskviewer': 'deskviewer'},
    author="Dincer Aslan",
    author_email="dinceraslan.com@gmail.com",
    description="A tool that allows remote computer control.",
    long_description=readme,
    long_description_content_type='text/x-rst',
    url="https://github.com/dinceraslancom/deskviewer",
    project_urls={
        'Source': 'https://github.com/dinceraslancom/deskviewer',
    },
    install_requires=requires,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.3",
    entry_points={
        "console_scripts": [
            "deskviewer.publish = deskviewer.server:main",
            "deskviewer.connect = deskviewer.client:main",
        ]
    },
)
